package outils;

import java.util.Iterator;

public class ListeChainee<Type> implements Liste<Type> {
	protected class ElementDeListe {
		public Type contenu;
		public ElementDeListe suivant, precedent;
		public ElementDeListe(Type element) {
			contenu = element;
			suivant = null;
			precedent = null;
		}
	}
	private class ParcoursALEndroit implements Parcours<Type> {
		private ElementDeListe courant;
		public ParcoursALEndroit() {
			courant = premier;
		}
		public boolean encore() {
			return courant != null;
		}
		public Type elementCourant() {
			return courant.contenu;
		}
		public void suivant() {
			courant = courant.suivant;
		}
		public void precedent() {
			if (courant == null)
				courant = dernier;
			else
				courant = courant.precedent;
		}
		public void supprime() {
			ElementDeListe aSupprimer = courant;
			courant = courant.suivant;
			supprimeElementDeListe(aSupprimer);
		}
	}
	protected ElementDeListe premier, dernier;
	protected int nbElem;
	public ListeChainee() {
		premier = null;
		dernier = null;
		nbElem = 0;
	}	
	public boolean estVide() {
		return nbElem == 0;
	}
	public int nombreDElements() {
		return nbElem;
	}
	public void ajoute(Type element) {
        ElementDeListe nouveau = new ElementDeListe(element);
        if (premier == null) {
            premier = nouveau;
        } else {
            dernier.suivant = nouveau;
            nouveau.precedent = dernier;
        }
        dernier = nouveau;
        nbElem++;
	}
    private void supprimeElementDeListe(ElementDeListe aSuppr) {
        if (aSuppr.precedent != null) {
            aSuppr.precedent.suivant = aSuppr.suivant;
        } else {
            premier = aSuppr.suivant;
        }
        if (aSuppr.suivant != null) {
            aSuppr.suivant.precedent = aSuppr.precedent;
        } else {
            dernier = aSuppr.precedent;
        }
        nbElem--;
        aSuppr.precedent = null;
        aSuppr.suivant = null;
    }
	private class RechercheDuDebut implements Strategie {
		public ElementDeListe courant;
		private int index;
		public RechercheDuDebut(int index) {
			courant = premier;
			this.index = index;
		}
		public void execute() {
	        while(index > 0) {
	            courant = courant.suivant;
	            index--;
	        }
		}
	}
	private class RechercheDeLaFin implements Strategie {
		public ElementDeListe courant;
		private int index;
		public RechercheDeLaFin(int index) {
			courant = dernier;
			this.index = nbElem - 1 - index;
		}
		public void execute() {
	        while(index > 0) {
	            courant = courant.precedent;
	            index--;
	        }
		}
	}
	public void retire(int index) {
		if (index < nbElem / 2) {
			RechercheDuDebut methodeDeRecherche = new RechercheDuDebut(index);
			methodeDeRecherche.execute();
			supprimeElementDeListe(methodeDeRecherche.courant);
		} else {
			RechercheDeLaFin methodeDeRecherche = new RechercheDeLaFin(index);
			methodeDeRecherche.execute();
			supprimeElementDeListe(methodeDeRecherche.courant);
		}
	}
	public Type element(int index) {
		if (index < nbElem / 2) {
			RechercheDuDebut methodeDeRecherche = new RechercheDuDebut(index);
			methodeDeRecherche.execute();
	        return methodeDeRecherche.courant.contenu;
		} else {
			RechercheDeLaFin methodeDeRecherche = new RechercheDeLaFin(index);
			methodeDeRecherche.execute();
	        return methodeDeRecherche.courant.contenu;
		}
	}
	public Parcours<Type> parcours() {
		return new ParcoursALEndroit();
	}
	public Iterator<Type> iterator() {
		return new AdaptateurDeParcours<Type>(new ParcoursALEndroit());
	}
}
